/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include <stdlib.h>
#include "ER-EPD037-1B.h"
#include "Debug.h"
#include "GUI_Paint.h"


/******************************************************************************
function :	Software reset
parameter:
******************************************************************************/
void EPD3IN7::EPD_Reset(void)
{
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
    EPD_RST_0;
    Dev->Dev_Delay_ms(200);
    EPD_RST_1;
    Dev->Dev_Delay_ms(200);
}

/******************************************************************************
function :	send command
parameter:
    Reg : Command register
******************************************************************************/
void EPD3IN7::EPD_SendCommand(UBYTE Reg)
{
    EPD_DC_0;
    EPD_CS_0;
    SPI_Write_Byte(Reg);
    EPD_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
void EPD3IN7::EPD_SendData(UBYTE Data)
{
    EPD_DC_1;
    EPD_CS_0;
    SPI_Write_Byte(Data);
    EPD_CS_1;
}

/******************************************************************************
function :	Wait until the busy_pin goes LOW
parameter:
******************************************************************************/
void EPD3IN7::EPD_WaitUntilIdle(void)
{
    while(EPD_BUSY_RD == 0) {      //HIGH: idle, LOW: busy
        Dev->Dev_Delay_ms(100);
    }
}

/******************************************************************************
function :	Turn On Display
parameter:
******************************************************************************/
void EPD3IN7::EPD_TurnOnDisplay(void)
{
    EPD_SendCommand(DISPLAY_REFRESH);
    EPD_WaitUntilIdle();
}


/******************************************************************************
function :	Initialize the e-Paper register
parameter:
******************************************************************************/
UBYTE EPD3IN7::EPD_Init(void)
{
    Dev->System_Init();
    EPD_Reset();

   EPD_SendCommand(0x01);  //vdh 11v vdl -11v vdhr 3.4v
   EPD_SendData(0x2f);
   EPD_SendData(0x04);//04=17v   05=18v  06
   EPD_SendData(0x30);
   EPD_SendData(0x30);//3F=15V  30=12V
   EPD_SendData(0x0F); //24=9.6V  1D=8.2V   12=6V  0f=5.4V
   
   EPD_SendCommand(0x06);
   EPD_SendData(0x17);
   EPD_SendData(0x17);
   EPD_SendData(0x17);
    EPD_WaitUntilIdle();

   
   EPD_SendCommand(0x00);
  EPD_SendData(0x1F);     //BW mode
 
  
   EPD_SendCommand(0x50);      
   EPD_SendData(0x20);         // 11  
   EPD_SendData(0x17);         //    

   EPD_SendCommand(0x52);      
   EPD_SendData(0x0A);          


    EPD_SendCommand(0x61);
    EPD_SendData(0xF0);  //240
    EPD_SendData(0x01);
    EPD_SendData(0xA0);  //416
	
    EPD_SendCommand(0x04);
    EPD_WaitUntilIdle();
   
    EPD_SendCommand(0x30);
    EPD_SendData(0x09); //set 100hz

   
   EPD_SendCommand(0x82);
   EPD_SendData(0x26); //vcom -2.0v

    EPD_WaitUntilIdle();

    
    return 0;
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void EPD3IN7::EPD_Clear(void)
{
    UWORD Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;

    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0X00);
        }
    }
    Dev->Dev_Delay_ms(2);


    EPD_SendCommand(DATA_START_TRANSMISSION_2);
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0X00);
        }
    }
    Dev->Dev_Delay_ms(2);

    // EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Sends the image buffer in RAM to e-Paper and displays
parameter:
******************************************************************************/
void EPD3IN7::EPD_Display(void)
{
    UDOUBLE Width, Height;
    Width = (EPD_WIDTH % 8 == 0)? (EPD_WIDTH / 8 ): (EPD_WIDTH / 8 + 1);
    Height = EPD_HEIGHT;
    
    EPD_SendCommand(DATA_START_TRANSMISSION_1);
    Dev->Dev_Delay_ms(2);
    for (UWORD j = 0; j < Height; j++) {
        for (UWORD i = 0; i < Width; i++) {
            EPD_SendData(0XFF);
        }
    }
    Dev->Dev_Delay_ms(2);
    
    EPD_SendCommand(DATA_START_TRANSMISSION_2);           
    Dev->Dev_Delay_ms(2);
    for (UDOUBLE j = 0; j < Height; j++) {
        for (UDOUBLE i = 0; i < Width; i++) {
            EPD_SendData(spiram->SPIRAM_RD_Byte(i + j * Width));
        }
    }
    
    EPD_TurnOnDisplay();
}

/******************************************************************************
function :	Enter sleep mode
parameter:
******************************************************************************/
void EPD3IN7::EPD_Sleep(void)
{   EPD_SendCommand(POWER_OFF);
    EPD_SendCommand(DEEP_SLEEP);
    EPD_SendData(0xa5);
}
